/*
 * GkSu -- Gtk+ Frontend to su
 * Copyright (C) 2002 Gustavo Noronha Silva
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include <gtk/gtk.h>

#include "gksutil.h"

/* Gettext */
#include <libintl.h>
#define _(String) gettext (String)
#define gettext_noop(String) (String)
/* Gettext */

/**
 * conv_from_utf8:
 * @msg: message to convert from UTF-8.
 * 
 * Converts a string from UTF-8 to the current locale. It is
 * mostly a wrapper to Glib's g_locale_from_utf8().
 *
 * Returns: a newly allocated with the converted string or
 * NULL on failure.
 *
 */
gchar *
conv_from_utf8 (gchar *msg)
{
  return g_locale_from_utf8 (msg, -1, NULL, NULL, NULL);
}

/**
 * conv_to_utf8:
 * @msg: message to convert to UTF-8.
 * 
 * Converts a string from the current locale to UTF-8. It is
 * mostly a wrapper to Glib's g_locale_to_utf8().
 *
 * Returns: a newly allocated with the converted string or
 * NULL on failure.
 *
 */
gchar *
conv_to_utf8 (gchar *msg)
{
  return g_locale_to_utf8 (msg, -1, NULL, NULL, NULL);
}

/**
 * gk_dialog:
 * @type: a #GtkMessageType (GTK_MESSAGE_{INFO,ERROR,WARNING})
 * @format: a printf()-like format for the label of the dialog
 * @Varargs: the actual variables.
 *
 * Shows 'msg' in a dialog box with an OK button 
 * This function is to be a helper for functions needing to
 * display some information to the user.
 * Notice that it will not let the program go on if the
 * user does not close the dialog.
*/
void 
gk_dialog (GtkMessageType type, gchar *format, ...)
{
  GtkWidget *diag_win;

  va_list ap;
  gchar *msg;

  va_start(ap, format);
  msg = g_strdup_vprintf(format, ap);
  va_end(ap);

  diag_win = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
				     type, GTK_BUTTONS_CLOSE,
				     msg);
			     
  gtk_signal_connect_object (GTK_OBJECT(diag_win), "delete_event",
			     GTK_SIGNAL_FUNC(gtk_main_quit), 
			     NULL);
  gtk_window_set_position (GTK_WINDOW(diag_win), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW(diag_win), FALSE);

  gtk_widget_show_all (diag_win);
  gtk_dialog_run (GTK_DIALOG(diag_win));

  g_free (msg);

  gtk_widget_destroy (diag_win);
}

/* Write all of buf, even if write(2) is interrupted. */
ssize_t
full_write (int d, const char *buf, size_t nbytes)
{
  ssize_t r, w = 0;

  /* Loop until nbytes of buf have been written. */
  while (w < nbytes) {
    /* Keep trying until write succeeds without interruption. */
    do {
      r = write(d, buf + w, nbytes - w);
    } while (r < 0 && errno == EINTR);

    if (r < 0) {
      return -1;
    }

    w += r;
  }

  return w;
}

void
copy (const char *fn, const char *dir)
{
  int in, out;
  int r;
  char *newfn;
  char buf[BUFSIZ];

  newfn = g_strdup_printf("%s/.Xauthority", dir);

  out = open(newfn, O_WRONLY | O_CREAT | O_EXCL);
  if (out == -1) 
    {
      if (errno == EEXIST)
	gk_dialog (GTK_MESSAGE_WARNING,
		   _("The X authority file i am trying to create for"
		     " root already exists! This is highly"
		     " suspicous!"));
      else
	gk_dialog (GTK_MESSAGE_ERROR,
		   _("Error copying '%s' to '%s': %s"),
		   fn, dir, strerror(errno));
    }
  
  in = open(fn, O_RDONLY);
  if (in == -1)
    gk_dialog (GTK_MESSAGE_ERROR,
	       _("Error copying '%s' to '%s': %s"),
	       fn, dir, strerror(errno));
  
  while ((r = read(in, buf, BUFSIZ)) > 0) 
    {
      if (full_write(out, buf, r) == -1)
	gk_dialog (GTK_MESSAGE_ERROR,
		   _("Error copying '%s' to '%s': %s"),
		   fn, dir, strerror(errno));
    }

  if (r == -1)
      gk_dialog (GTK_MESSAGE_ERROR,
		 _("Error copying '%s' to '%s': %s"),
		 fn, dir, strerror(errno));
}
